@@ -52,6 +52,10 @@ import click
               help='Default minimization strategy')
 @click.option('--hesse/--no-hesse', default=False, show_default=True,
               help='Evaluate errors using Hesse.')
+@click.option('--improve', type=int, default=0, show_default=True,
+              help='Execute improve after each minimization')
+@click.option('--minimizer_offset', type=int, default=1, show_default=True,
+              help='Enable minimizer offsetting')
 @click.option('--minos/--no-minos', default=False, show_default=True,
               help='Evaluate errors using Minos.')
 @click.option('--print_level', type=int, default=-1, show_default=True,
@@ -224,6 +228,10 @@ def likelihood_fit(**kwargs):
               help='Maximum number of Minuit iterations')
 @click.option('--optimize', type=int, default=2, show_default=True,
               help='Optimize constant terms')
+@click.option('--improve', type=int, default=0, show_default=True,
+              help='Execute improve after each minimization')
+@click.option('--minimizer_offset', type=int, default=1, show_default=True,
+              help='Enable minimizer offsetting')
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood')
 @click.option('--batch_mode/--no-batch', default=False, show_default=True,
