@@ -77,6 +77,10 @@ import click
               help='Maximum number of Minuit iterations.')
 @click.option('--optimize', type=int, default=2, show_default=True,
               help='Optimize constant terms.')
+@click.option('--improve', type=int, default=0, show_default=True,
+              help='Execute improve after each minimization')
+@click.option('--minimizer_offset', type=int, default=1, show_default=True,
+              help='Enable minimizer offsetting')
 @click.option('--offset/--no-offset', default=True, show_default=True,
               help='Offset likelihood.')
 @click.option('-v', '--verbosity', default='INFO', show_default=True,
