@@ -132,7 +132,7 @@ class AsimovGenerator(AnalysisObject):
             "nll_options"         : self.nll_commands,
             "snapshot_names"      : snapshot_names
         }
-        self.model.generate_asimov(**kwargs)
+        return self.model.generate_asimov(**kwargs)
         
     def generate_standard_asimov(self, asimov_types:Optional[Union[List[AsimovType], List[int]]]=None,
                                  asimov_names:Optional[Union[List[str], str]]=None,
