@@ -40,7 +40,8 @@ class AsymptoticCLs(AnalysisObject):
                  num_cpu:int=1, offset:bool=True, optimize:int=2,
                  fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1,
                  max_iters:int=-1, constrain_nuis:bool=True, batch_mode:bool=False,
-                 int_bin_precision:float=-1., verbosity:Optional[Union[int, str]]="INFO", **kwargs):
+                 int_bin_precision:float=-1., improve:int=0, minimizer_offset:int=1,
+                 verbosity:Optional[Union[int, str]]="INFO", **kwargs):
         """
         args:
             precision: precision in mu that defines iterative cutoff
@@ -66,6 +67,8 @@ class AsymptoticCLs(AnalysisObject):
             'num_cpu': num_cpu,
             'offset': offset,
             'optimize': optimize,
+            'improve': improve,
+            'minimizer_offset': minimizer_offset,
             'eigen': False,
             'print_level': print_level,
             'timer': timer,
