@@ -33,9 +33,12 @@ class WSArgument(DescriptiveEnum):
     MUTABLE                           = (15, "Mutable parameters (Nuisance parameters, global observables, POIs and floating auxiliary"
                                              " parameters)", True)
     
-    def __init__(self, enum_id:int, description:str="", is_variable:bool=False):
-        super().__init__(enum_id=enum_id, description=description)
-        self.is_variable = is_variable
+    def __new__(cls, value:int, description:str="", is_variable:bool=False):
+        obj = object.__new__(cls)
+        obj._value_ = value
+        obj.description = description
+        obj.is_variable = is_variable
+        return obj
 
 class SetValueMode(GeneralEnum):
     UNCHANGED = 0
