@@ -27,8 +27,10 @@ class ExtendedMinimizer(AbstractObject):
         'hesse': 0,
         'minos': 0,
         'scan': 0,
+        'improve': 0,
         'num_ee': 5,
         'do_ee_wall': 1,
+        'minimizer_offset': 1,
         'retry': 1,
         'eigen': 0,
         'reuse_minimizer': 0,
@@ -52,7 +54,9 @@ class ExtendedMinimizer(AbstractObject):
         return self._config
     
     def __init__(self, minimizer_name:str, pdf:"ROOT.RooAbsPdf", data:"ROOT.RooAbsData",
-                 minimizer_options:Optional[Dict]=None, verbosity:Optional[Union[int, str]]="INFO"):
+                 minimizer_options:Optional[Dict]=None,
+                 verbosity:Optional[Union[int, str]]="INFO",
+                 **kwargs):
         super().__init__(verbosity=verbosity)
         """
             Minimizr Options:
@@ -86,7 +90,7 @@ class ExtendedMinimizer(AbstractObject):
         
         self.configure_default_minimizer_options()
         
-        self.stdout.info('INFO: Created ExtendedMinimizer("{}") instance'.format(self.name))
+        self.stdout.info(f'INFO: Created ExtendedMinimizer("{self.name}") instance')
         
     @property
     def data(self):
@@ -210,6 +214,7 @@ class ExtendedMinimizer(AbstractObject):
         if (self.minimizer is None) or (not self.config['reuse_minimizer']):
             self.configure_default_minimizer_options()
             self.minimizer = ROOT.RooMinimizer(self.nll)
+            self.minimizer.setOffsetting(self.config['minimizer_offset'])
             self.minimizer.setPrintLevel(self.config['print_level'])
             self.minimizer.optimizeConst(self.config['opt_const'])
             self.minimizer.setMinimizerType(self.config['minimizer_type'])
@@ -328,6 +333,10 @@ class ExtendedMinimizer(AbstractObject):
         else:
             status = self.robust_minimize()
 
+        # Evaluate errors with improve
+        if self.config['improve']:
+            self.minimizer.improve()
+            
         # Evaluate errors with Hesse
         if self.config['hesse']:
             self.minimizer.hesse()
