@@ -27,6 +27,7 @@ from quickstats.interface.root import TH1, RooDataSet, RooArgSet, RooMsgService
 from quickstats.interface.root import roofit_extension as rf_ext
 from quickstats.interface.root.roofit_extension import get_str_data
 from quickstats.components.basics import WSArgument, SetValueMode
+from .extended_minimizer import ExtendedMinimizer
 
 class ExtendedModel(AbstractObject):
     
@@ -58,7 +59,7 @@ class ExtendedModel(AbstractObject):
                  data_name:Optional[str]="combData", snapshot_name:Optional[Union[List[str], str]]=None,
                  binned_likelihood:bool=True, tag_as_measurement:Optional[str]=None,
                  fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
-                 load_extension:bool=True, verbosity:Optional[Union[int, str]]="INFO"):
+                 load_extension:bool=True, minimizer_cls=None, verbosity:Optional[Union[int, str]]="INFO"):
         super().__init__(verbosity=verbosity)
         self.fname = fname
         self.ws_name = ws_name
@@ -70,7 +71,11 @@ class ExtendedModel(AbstractObject):
         self.fix_cache = fix_cache
         self.fix_multi = fix_multi
         self.interpolation_code = interpolation_code
-        
+        if minimizer_cls is None:
+            self.minimizer_cls = ExtendedMinimizer
+        else:
+            self.minimizer_cls = minimizer_cls
+            
         quickstats.load_corelib()
         if load_extension:
             self.load_extension()
@@ -1005,10 +1010,9 @@ class ExtendedModel(AbstractObject):
             ws.saveSnapshot(nom_nuis_name, all_nuis)
         
         if do_fit:
-            from quickstats.components import ExtendedMinimizer
             if dataset is None:
                 dataset = self.data
-            minimizer = ExtendedMinimizer("Minimizer", self.pdf, dataset)
+            minimizer = self.minimizer_cls("Minimizer", self.pdf, dataset, workspace=self.workspace)
             if minimizer_options is None:
                 minimizer_options = copy.deepcopy(minimizer._DEFAULT_MINIMIZER_OPTION_)
             if nll_options is None:
