@@ -9,6 +9,14 @@ import numpy as np
 import ROOT
 from quickstats.maths.numerics import pretty_value
 from quickstats.components.basics import WSArgument
+from quickstats import DescriptiveEnum
+
+class FitMode(DescriptiveEnum):
+    HYBRID  = (0, "Execute both conditional and unconditional fits for a given set of POIs")
+    UNCOND  = (1, "Execute unconditional fit for a given set of POIs")
+    COND    = (2, "Execute conditional fit with specified conditional values for a given set of POIs")
+    NOMINAL = (3, "Execute fit without any POIs")
+    NOFIT   = (4, "Do not perform only fit")
 
 def load_cache(fname:str):
     if (fname is not None) and os.path.exists(fname):
@@ -55,8 +63,9 @@ class Likelihood(AnalysisObject):
         return poi_val
         
     def nll_fit(self, poi_val:Optional[Union[Dict[str, float], float]]=None,
-                mode:int=2, snapshot_name:Optional[str]=AnalysisObject.kCurrentSnapshotName,
+                mode:Union[int, FitMode]=FitMode.COND, snapshot_name:Optional[str]=AnalysisObject.kCurrentSnapshotName,
                 do_minos:bool=False):
+        mode = FitMode.parse(mode)
         if not isinstance(self.poi, ROOT.RooArgSet):
             pois = ROOT.RooArgSet(self.poi)
         else:
@@ -85,7 +94,7 @@ class Likelihood(AnalysisObject):
         muhat_errhi = {}
         
         # no poi fit
-        if mode == 3:
+        if mode == FitMode.NOMINAL:
             # restore initial parameter values after each fit
             self.load_snapshot(snapshot_name)
             if len(pois) > 0:
@@ -103,7 +112,7 @@ class Likelihood(AnalysisObject):
             fit_time = time.time() - start_time
             
         # unconditional nll
-        if mode in [0, 1, 4]:
+        if mode in [FitMode.HYBRID, FitMode.UNCOND, FitMode.NOFIT]:
             # restore initial parameter values after each fit
             self.load_snapshot(snapshot_name)
             # fix other pois if they are not studied
@@ -114,7 +123,7 @@ class Likelihood(AnalysisObject):
                 val = poi_val[poi.GetName()]
                 if val is not None:
                     poi.setVal(val)
-            if mode == 4:
+            if mode == FitMode.NOFIT:
                 self.minimizer.create_nll()
                 fit_status_uncond = None
                 roofit_result['uncond'] = None
@@ -130,7 +139,7 @@ class Likelihood(AnalysisObject):
                 muhat_errhi[poi_name] = poi.getErrorHi()
         fit_time_uncond = time.time() - start_time
         # conditional nll
-        if mode in [0, 2]:
+        if mode in [FitMode.HYBRID, FitMode.COND]:
             # restore initial parameter values after each fit
             self.load_snapshot(snapshot_name)
             # fix other pois if they are not studied
@@ -152,7 +161,7 @@ class Likelihood(AnalysisObject):
         
         fit_result = {}
         self.stdout.info("INFO: NLL evaluation completed with")
-        if mode in [0, 1, 4]:
+        if mode in [FitMode.HYBRID, FitMode.UNCOND, FitMode.NOFIT]:
             best_fit_str = ", ".join([f"{name}={value:.5f}" for name, value in muhat.items()])
             self.stdout.info("best fit : ".rjust(15) + f"{best_fit_str}")
             self.stdout.info("uncond NLL = ".rjust(15) + f"{nll_uncond:.5f}")
@@ -164,7 +173,7 @@ class Likelihood(AnalysisObject):
                 'status': fit_status_uncond,
                 'time': fit_time_uncond
             }
-        if mode in [0, 2]:
+        if mode in [FitMode.HYBRID, FitMode.COND]:
             mu = {name:pretty_value(value) for name, value in poi_val.items()}
             mu_str = ", ".join([f"{name}={pretty_value(value)}" for name, value in poi_val.items()])
             self.stdout.info("mu : ".rjust(15) + f"{mu_str}")
@@ -181,7 +190,7 @@ class Likelihood(AnalysisObject):
                 'time': fit_time_cond
             }
             
-        if mode == 0:
+        if mode == FitMode.HYBRID:
             pnll = nll_cond - nll_uncond
             qmu  = 2*pnll
             self.stdout.info("PNLL = ".rjust(15) + "{:.5f}".format(nll_cond - nll_uncond))
@@ -210,7 +219,7 @@ class Likelihood(AnalysisObject):
                 fit_result['significance'] = None
                 fit_result['pvalue'] = None
                 
-        if mode != 3:
+        if mode != FitMode.NOMINAL:
             self.stdout.info("time = ".rjust(15) + \
                              "(uncond_fit) {:.3f}, (cond_fit) {:.3f}".format(fit_time_uncond, fit_time_cond))
             fit_result['total_time'] = fit_time_uncond + fit_time_cond
@@ -231,7 +240,7 @@ class Likelihood(AnalysisObject):
         return fit_result
         
     def evaluate_nll(self, poi_val:Optional[Union[Dict[str, float], float]]=None,
-                     mode:int=2, snapshot_name:Optional[str]="currentSnapshot"):
+                     mode:Union[int, FitMode]=FitMode.COND, snapshot_name:Optional[str]="currentSnapshot"):
         """Evalute post-fit NLL value
         
         Arguments:
@@ -247,12 +256,12 @@ class Likelihood(AnalysisObject):
                 Name of snapshot to load before fitting.
         """
         fit_result = self.nll_fit(poi_val=poi_val, mode=mode, snapshot_name=snapshot_name)
-        
-        if mode == 0:
+        mode = FitMode.parse(mode)
+        if mode == FitMode.HYBRID:
             return fit_result['pnll']
-        if mode == 1:
+        elif mode == FitMode.UNCOND:
             return fit_result['uncond_fit']['nll']
-        if mode == 2:
+        elif mode == FitMode.COND:
             return fit_result['cond_fit']['nll']
-        if mode == 3:
+        elif mode == FitMode.NOMINAL:
             return fit_result['postfit_nll']
\ No newline at end of file
