@@ -83,7 +83,8 @@ class NuisanceParameterPull(object):
                    num_cpu:int=1, offset:bool=True, optimize:int=2, eigen:bool=False,
                    fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
                    max_iters:int=-1, batch_mode:bool=False, int_bin_precision:float=-1.,
-                   outdir:str='pulls', cache=True, logfile_path=None, **kwargs):
+                   outdir:str='pulls', cache=True, logfile_path=None,
+                   minimizer_cls=None, **kwargs):
         # create output directory if not exists
         if not os.path.exists(outdir):
             os.makedirs(outdir, exist_ok=True)        
@@ -131,7 +132,9 @@ class NuisanceParameterPull(object):
         print('INFO: Computing error for parameter "{}" at {}'.format(nuip.GetName(), nuip.getVal()))
         
         print("INFO: Making ExtendedMinimizer for unconditional fit")
-        minimizer = ExtendedMinimizer("minimizer", model.pdf, model.data)
+        if minimizer_cls is None:
+            minimizer_cls = ExtendedMinimizer
+        minimizer = minimizer_cls("minimizer", model.pdf, model.data, workspace=model.workspace)
         print("INFO: Starting minimization")
         nll_commands = [ROOT.RooFit.NumCPU(num_cpu, 3),
                         #ROOT.RooFit.Constrain(model.nuisance_parameters),
@@ -278,7 +281,7 @@ class NuisanceParameterPull(object):
                   eigen:bool=False, strategy:int=0, print_level:int=1, fix_cache:bool=True, fix_multi:bool=True,
                   offset:bool=True, optimize:int=2, parameter:str="", max_calls:int=-1, max_iters:int=-1,
                   batch_mode:bool=False, int_bin_precision:float=-1.,parallel:int=0, cache:bool=True,
-                  exclude:str='', constrained_only:bool=True, **kwargs):
+                  exclude:str='', constrained_only:bool=True, minimizer_cls=None, **kwargs):
         
         # configure default minimizer options
         ExtendedMinimizer._configure_default_minimizer_options(minimizer_type, minimizer_algo,
@@ -321,7 +324,8 @@ class NuisanceParameterPull(object):
                      repeat(strategy), repeat(print_level), repeat(num_cpu), repeat(offset),
                      repeat(optimize), repeat(eigen), repeat(fix_cache), repeat(fix_multi),
                      repeat(max_calls), repeat(max_iters), repeat(batch_mode),
-                     repeat(int_bin_precision), repeat(outdir), repeat(cache), logfile_paths)
+                     repeat(int_bin_precision), repeat(outdir), repeat(cache), logfile_paths,
+                     repeat(minimizer_cls))
         common_utils.execute_multi_tasks(self._run_pulls, *arguments, parallel=parallel)
 
         
