@@ -4,8 +4,10 @@
 ###############################################################################
 import os
 import re
+import copy
 import json
 import time
+import yaml
 from typing import Optional, Union, List, Dict
 
 import numpy as np
@@ -62,6 +64,8 @@ class XMLWSModifier(XMLWSBase):
                 self.parse_config_xml(source)
             elif ext == ".json":
                 self.parse_config_json(source)
+            elif ext == ".yaml":
+                self.parse_config_yaml(source)
             else:
                 raise ValueError(f"unsupported file format: {ext}")
         elif isinstance(source, dict):
@@ -73,10 +77,11 @@ class XMLWSModifier(XMLWSBase):
                 raise RuntimeError("\"add product terms\" is only supported after ROOT 6.26")
             
     def parse_config_dict(self, source:Dict):
+        source = copy.deepcopy(source)
         _actions = source.pop("actions", {})
         config = {
-            "input_file"         : source.pop("input_file"),
-            "output_file"        : source.pop("output_file"),
+            "input_file"         : source.pop("input_file", "dummy"),
+            "output_file"        : source.pop("output_file", "dummy"),
             "model_name"         : source.pop("model_name", None),
             "workspace_name"     : source.pop("workspace_name", None),
             "model_config_name"  : source.pop("model_config_name", None),
@@ -92,8 +97,8 @@ class XMLWSModifier(XMLWSBase):
         }
         if source:
             unknown_attributes = list(source)
-            self.stdout.warning(f"WARNING: Unknown attributes received: {', '.join(unknown_attributes)}. "
-                                "Ignoring...")
+            self.stdout.warning(f"WARNING: The following unrecognized configuration attribute(s) will be ignored: "
+                                f"{', '.join(unknown_attributes)}")
         actions = {
             "map"        : _actions.pop("map", []),
             "define"     : _actions.pop("define", []),
@@ -140,6 +145,11 @@ class XMLWSModifier(XMLWSBase):
         with open(filename, 'r') as f:
             source = json.load(f)
         self.parse_config_dict(source)
+        
+    def parse_config_yaml(self, filename:str):
+        with open(filename, 'r') as f:
+            source = yaml.safe_load(f)
+        self.parse_config_dict(source)        
     
     def parse_config_xml(self, filename:str):
         root = self._xml_to_dict(filename)
@@ -289,7 +299,7 @@ class XMLWSModifier(XMLWSBase):
             flag |= self.implement_objects(ws_tmp)
             # nothing is done
             if not flag:
-                self.stdout.info("INFO: No objects are defined.")
+                self.stdout.info("INFO: No new objects are defined.")
 
             # need to import datasets first so the variables are also imported into the workspace
             self.import_datasets(ws_orig, ws_tmp)
