@@ -113,7 +113,7 @@ class PathManager:
         return directory
     
     def get_resolved_file(self, file_name:str, check_exist:bool=False,
-                          **parameters):
+                          subdirectory:Optional[str]=None, **parameters):
         if file_name not in self.files:
             raise KeyError(f'unrecognized file name "{file_name}"')
         file = self.files[file_name]
@@ -121,14 +121,16 @@ class PathManager:
             basename = file.resolve_basename(**parameters)
         else:
             basename = file.basename
+        if subdirectory is None:
+            subdirectory = ""
         if file.dirname is not None:
             dirname  = self.get_directory(file.dirname)
-            fullname = os.path.join(dirname, basename)
+            fullname = os.path.join(dirname, subdirectory, basename)
         else:
             if self.base_path is not None:
-                fullname = os.path.join(self.base_path, basename)
+                fullname = os.path.join(self.base_path, subdirectory, basename)
             else:
-                fullname = basename
+                fullname = os.path.join(subdirectory, basename)
         if check_exist:
             if not os.path.exists(fullname):
                 raise FileNotFoundError(f'file "{fullname}" does not exist')
