@@ -74,7 +74,7 @@ class TH1(TObject):
         if len(neg_bins) > 0:
             for i in neg_bins:
                 h.SetBinContent(int(i), 0.)
-        return neg_bin_indices, neg_bin_values
+        return neg_bin_indices[0], neg_bin_values
     
     @staticmethod
     def GetBinContentArray(h, dtype:str='double', underflow_bin:int=0, overflow_bin:int=0):
