@@ -15,7 +15,9 @@ root_datatypes = ["bool", "Bool_t", "Byte_t", "char", "char*", "Char_t",
 
 uproot_datatypes = ["double", "float", "int", "int64_t", "char*", "int32_t", "uint64_t"]
 
-def get_default_library():
+def get_default_library(custom_columns:bool=False):
+    if custom_columns:
+        return "root"
     try:
         import uproot
         has_uproot = True
@@ -38,7 +40,7 @@ def downcast_dataframe(df):
     df[fcols] = df[fcols].apply(pd.to_numeric, downcast='float')
     df[icols] = df[icols].apply(pd.to_numeric, downcast='integer')
 
-def array2root(array_data:Dict[str, np.ndarray], fname:str, tree_name:str,
+def array2root(array_data:Dict[str, np.ndarray], fname:str, treename:str,
                library:str="auto", multithread:bool=True):
     if library.lower() == "auto":
         library = get_default_library()
@@ -60,15 +62,15 @@ def array2root(array_data:Dict[str, np.ndarray], fname:str, tree_name:str,
             snapshot_templates = tuple(snapshot_templates)
             import ROOT
             df = ROOT.RDF.MakeNumpyDataFrame(array_data)
-            df.Snapshot.__getitem__(snapshot_templates)(tree_name, fname, columns)
+            df.Snapshot.__getitem__(snapshot_templates)(treename, fname, columns)
     elif library == "uproot":
         import uproot
         from packaging import version
         uproot_version = uproot.__version__
         if version.parse(uproot_version) < version.parse("4.2.0"):
-            raise RuntimeError("uproot version too old (<4.2.0)")
+            raise RuntimeError("uproot version too old (requires 4.2.0+)")
         file = uproot.recreate(fname)
-        file[tree_name] = array_data
+        file[treename] = array_data
         file.close()
     else:
         raise RuntimeError(f'unknown library "{library}" for root data conversion')            
@@ -84,12 +86,22 @@ def dataframe2numpy(df:"pandas.DataFrame", columns:Optional[List[str]]=None):
         arrays[column] = arrays[column].astype(df[column].dtype)
     return arrays 
 
+def numpy2dataframe(array_data:Dict[str, np.ndarray]):
+    array_shallow_copy = {**array_data}
+    for key, array in array_data.items():
+        if (array.ndim > 1) and (array.dtype != object):
+            array_shallow_copy[key] = list(array)
+    import pandas as pd
+    df = pd.DataFrame(array_shallow_copy)
+    return df
 
-def dataframe2root(df:"pandas.DataFrame", fname:str, tree_name:str,
+array2dataframe = numpy2dataframe
+
+def dataframe2root(df:"pandas.DataFrame", fname:str, treename:str,
                    columns:Optional[List[str]]=None,
                    library:str="auto", multithread:bool=True):
     array_data = dataframe2numpy(df, columns)
-    array2root(array_data, fname, tree_name, library=library,
+    array2root(array_data, fname, treename, library=library,
                multithread=multithread)
     
 def uproot_get_standard_columns(uproot_tree):
@@ -97,77 +109,88 @@ def uproot_get_standard_columns(uproot_tree):
     columns = list(typenames.keys())
     column_types = list(typenames.values())
     return np.array(columns)[np.where(np.isin(column_types, uproot_datatypes))]
+
+def rdf2numpy(rdf, columns:Union[Dict[str, str], List[str]]=None,
+              cut:Optional[str]=None, convert_vectors:bool=True,
+              remove_non_standard_types:bool=True):
+    if cut is not None:
+        rdf = rdf.Filter(cut)     
+    rename_columns = {}
+    if columns is None:
+        save_columns = [str(name) for name in rdf.GetColumnNames()]
+    elif isinstance(columns, dict):
+        save_columns = []
+        for column_name, definition in columns.items():
+            if column_name == definition:
+                save_columns.append(column_name)
+                continue
+            if rdf.HasColumn(column_name):
+                new_column_name = f"var_{uuid.uuid4().hex}"
+                rename_columns[new_column_name] = column_name
+                column_name = new_column_name
+            rdf = rdf.Define(column_name, definition)
+            save_columns.append(column_name)
+    else:
+        save_columns = list(columns)
+    if remove_non_standard_types:
+        column_types = np.array([rdf.GetColumnType(column_name) for column_name in save_columns])
+        save_columns = list(np.array(save_columns)[np.where(np.isin(column_types, root_datatypes))])
+        rename_columns = {k:v for k,v in rename_columns.items() if k in save_columns}
+    vector_columns = []
+    if convert_vectors:
+        vector_columns_tmp = []
+        for column_name in save_columns:
+            column_type = rdf.GetColumnType(column_name)
+            if column_type.count("ROOT::VecOps::RVec") == 1:
+                vector_columns_tmp.append(column_name)
+        if len(vector_columns_tmp) > 0:
+            import quickstats
+            quickstats.load_processor_methods()
+            for column_name in vector_columns_tmp:
+                new_column_name = f"var_{uuid.uuid4().hex}"
+                if column_name in save_columns:
+                    save_columns = [new_column_name if name == column_name else name for name in save_columns]
+                # mapped twice
+                if column_name in rename_columns:
+                    rename_columns[new_column_name] = rename_columns.pop(column_name)
+                else:
+                    rename_columns[new_column_name] = column_name
+                rdf = rdf.Define(new_column_name, f"RVec2Vec({column_name})")
+                vector_columns.append(new_column_name)
+
+    available_columns = [str(name) for name in rdf.GetColumnNames()]
+    missing_columns = np.setdiff1d(save_columns, available_columns)
+    if len(missing_columns) > 0:
+        raise RuntimeError(f'missing column(s): {", ".join(missing_columns)}')
+    result = rdf.AsNumpy(save_columns)
+    for vector_column in vector_columns:
+        # not the most efficient way, but easiest
+        numpy_array = np.array([np.array(v.data()) for v in result[vector_column]], dtype=object)
+        # in case it't array of regular size
+        if len(numpy_array) and (numpy_array[0].dtype == object):
+            result[vector_column] = np.array([np.array(v.data()) for v in result[vector_column]])
+        else:
+            result[vector_column] = numpy_array
+    for old_column, new_column in rename_columns.items():
+        result[new_column] = result.pop(old_column)
+    # reorder the columns to match the order given by the user
+    if columns is not None:
+        result = {column: result[column] for column in columns if column in result}
+    return result
     
 def root2numpy(filename:Union[str, List[str]], treename:str,
                columns:Union[Dict[str, str], List[str]]=None,
                cut:Optional[str]=None, convert_vectors:bool=True,
-               remove_non_standard_types:bool=True, library:str="auto"):
+               remove_non_standard_types:bool=True,
+               library:str="auto"):
     if library.lower() == "auto":
-        library = get_default_library()
+        library = get_default_library(custom_columns=isinstance(columns,dict))
     if library.lower() == "root":
         import ROOT
         rdf = ROOT.RDataFrame(treename, filename)
-        if cut is not None:
-            rdf = rdf.Filter(cut)
-        rename_columns = {}
-        if columns is None:
-            save_columns = [str(name) for name in rdf.GetColumnNames()]
-        elif isinstance(columns, dict):
-            save_columns = []
-            for column_name, definition in columns.items():
-                if column_name == definition:
-                    save_columns.append(column_name)
-                    continue
-                if rdf.HasColumn(column_name):
-                    new_column_name = f"var_{uuid.uuid4().hex}"
-                    rename_columns[new_column_name] = column_name
-                    column_name = new_column_name
-                rdf = rdf.Define(column_name, definition)
-                save_columns.append(column_name)
-        else:
-            save_columns = list(columns)
-
-        if remove_non_standard_types:
-            column_types = np.array([rdf.GetColumnType(column_name) for column_name in save_columns])
-            save_columns = list(np.array(save_columns)[np.where(np.isin(column_types, root_datatypes))])
-        
-        vector_columns = []
-        if convert_vectors:
-            vector_columns_tmp = []
-            for column_name in save_columns:
-                column_type = rdf.GetColumnType(column_name)
-                if column_type.count("ROOT::VecOps::RVec") == 1:
-                    vector_columns_tmp.append(column_name)
-            if len(vector_columns_tmp) > 0:
-                import quickstats
-                quickstats.load_processor_methods()
-                for column_name in vector_columns_tmp:
-                    new_column_name = f"var_{uuid.uuid4().hex}"
-                    if column_name in save_columns:
-                        save_columns = [new_column_name if name == column_name else name for name in save_columns]
-                    # mapped twice
-                    if column_name in rename_columns:
-                        rename_columns[new_column_name] = rename_columns.pop(column_name)
-                    else:
-                        rename_columns[new_column_name] = column_name
-                    rdf = rdf.Define(new_column_name, f"RVec2Vec({column_name})")
-                    vector_columns.append(new_column_name)
-                                          
-        available_columns = [str(name) for name in rdf.GetColumnNames()]
-        missing_columns = np.setdiff1d(save_columns, available_columns)
-        if len(missing_columns) > 0:
-            raise RuntimeError(f'missing column(s): {", ".join(missing_columns)}')
-        result = rdf.AsNumpy(save_columns)
-        for vector_column in vector_columns:
-            # not the most efficient way, but easiest
-            result[vector_column] = np.array([np.array(v.data()) for v in result[vector_column]], dtype=object)
-        for old_column, new_column in rename_columns.items():
-            result[new_column] = result.pop(old_column)
-        # reorder the columns to match the order given by the user
-        if columns is not None:
-            result = {column: result[column] for column in columns}
-        return result
-    
+        return rdf2numpy(rdf, columns=columns, cut=cut,
+                         convert_vectors=convert_vectors,
+                         remove_non_standard_types=remove_non_standard_types)
     elif library.lower() == "uproot":
         if isinstance(columns, dict):
             raise RuntimeError('defining new columns are not supported when using "uproot" as the library')
@@ -201,6 +224,8 @@ def root2numpy(filename:Union[str, List[str]], treename:str,
             return result
     else:
         raise RuntimeError(f'unknown library "{library}" for root data conversion')
+
+root2array = root2numpy
         
 def root2dataframe(filename:Union[str, List[str]], treename:str,
                    columns:Union[Dict[str, str], List[str]]=None,
@@ -209,14 +234,13 @@ def root2dataframe(filename:Union[str, List[str]], treename:str,
                    downcast:bool=True,
                    library:str="auto"):
     if library.lower() == "auto":
-        library = get_default_library()
+        library = get_default_library(custom_columns=isinstance(columns,dict))
     if library.lower() == "root":
         numpy_data = root2numpy(filename, treename, columns=columns, cut=cut,
                                 convert_vectors=True,
                                 remove_non_standard_types=remove_non_standard_types,
                                 library=library)
-        import pandas as pd
-        result = pd.DataFrame(numpy_data)
+        result = numpy2dataframe(numpy_data)
     elif library.lower() == "uproot":
         if isinstance(columns, dict):
             raise RuntimeError('defining new columns are not supported when using "uproot" as the library')
